from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.views.decorators.http import require_POST

from django.contrib.auth.decorators import login_required

from user_messages.forms import MessageReplyForm, NewMessageForm, NewMessageFormMultiple
from user_messages.models import Thread


@login_required
def inbox(request, template_name="user_messages/inbox.html"):
    threads_all = Thread.ordered(Thread.objects.inbox(request.user))
    threads_unread = Thread.ordered(Thread.objects.unread(request.user))
    return render_to_response(template_name, {
        "threads_all": threads_all,
        "threads_unread" : threads_unread,
    }, context_instance=RequestContext(request))


@login_required
def thread_detail(request, thread_id,
    template_name="user_messages/thread_detail.html",
    form_class=MessageReplyForm):
    qs = Thread.objects.filter(userthread__user=request.user).distinct()
    thread = get_object_or_404(qs, pk=thread_id)
    if request.method == "POST":
        form = form_class(request.POST, user=request.user, thread=thread)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse("messages_inbox"))
    else:
        form = form_class(user=request.user, thread=thread)
        thread.userthread_set.filter(user=request.user).update(unread=False)
    return render_to_response(template_name, {
        "thread": thread,
        "form": form
    }, context_instance=RequestContext(request))


@login_required
def message_create(request, user_id=None,
    template_name="user_messages/message_create.html",
    form_class=None, multiple=False):
    if form_class is None:
        if multiple:
            form_class = NewMessageFormMultiple
        else:
            form_class = NewMessageForm
    
    if user_id is not None:
        user_id = [int(user_id)]
    elif "to_user" in request.GET and request.GET["to_user"].isdigit():
        user_id = map(int, request.GET.getlist("to_user"))
    if not multiple and user_id:
        user_id = user_id[0]
    initial = {"to_user": user_id}
    if request.method == "POST":
        form = form_class(request.POST, user=request.user, initial=initial)
        if form.is_valid():
            msg = form.save()
            return HttpResponseRedirect(msg.get_absolute_url())
    else:
        form = form_class(user=request.user, initial=initial)
    return render_to_response(template_name, {
        "form": form
    }, context_instance=RequestContext(request))


@login_required
@require_POST
def thread_delete(request, thread_id):
    qs = Thread.objects.filter(userthread__user=request.user)
    thread = get_object_or_404(qs, pk=thread_id)
    thread.userthread_set.filter(user=request.user).update(deleted=True)
    return HttpResponseRedirect(reverse("messages_inbox"))



# -*- coding: utf-8 -*-
#########################################################################
#
# Copyright (C) 2016 OSGeo
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################

from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sent_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('content', models.TextField()),
                ('sender', models.ForeignKey(related_name='sent_messages', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('sent_at',),
            },
        ),
        migrations.CreateModel(
            name='Thread',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('subject', models.CharField(max_length=150)),
            ],
        ),
        migrations.CreateModel(
            name='UserThread',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('unread', models.BooleanField()),
                ('deleted', models.BooleanField()),
                ('thread', models.ForeignKey(to='user_messages.Thread')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='thread',
            name='users',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, through='user_messages.UserThread'),
        ),
        migrations.AddField(
            model_name='message',
            name='thread',
            field=models.ForeignKey(related_name='messages', to='user_messages.Thread'),
        ),
    ]



from django import forms

from django.contrib.auth import get_user_model

from user_messages.models import Message


class NewMessageForm(forms.Form):

    to_user = forms.ModelChoiceField(label="To",queryset=get_user_model().objects.exclude(username='AnonymousUser'))    
    subject = forms.CharField()
    content = forms.CharField(widget=forms.Textarea)
    
    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super(NewMessageForm, self).__init__(*args, **kwargs)
        if self.initial.get("to_user") is not None:
            qs = self.fields["to_user"].queryset.filter(pk=self.initial["to_user"])
            self.fields["to_user"].queryset = qs
    
    def save(self):
        data = self.cleaned_data
        return Message.objects.new_message(self.user, [data["to_user"]],
            data["subject"], data["content"])


class NewMessageFormMultiple(forms.Form):

    to_user = forms.ModelMultipleChoiceField(label="To",queryset=get_user_model().objects.exclude(username='AnonymousUser')) 
    subject = forms.CharField()
    content = forms.CharField(widget=forms.Textarea)
    
    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super(NewMessageFormMultiple, self).__init__(*args, **kwargs)
        if self.initial.get("to_user") is not None:
            qs = self.fields["to_user"].queryset.filter(pk__in=self.initial["to_user"])
            self.fields["to_user"].queryset = qs
    
    def save(self):
        data = self.cleaned_data
        return Message.objects.new_message(self.user, data["to_user"],
            data["subject"], data["content"])


class MessageReplyForm(forms.Form):
    
    content = forms.CharField(widget=forms.Textarea)
    
    def __init__(self, *args, **kwargs):
        self.thread = kwargs.pop("thread")
        self.user = kwargs.pop("user")
        super(MessageReplyForm, self).__init__(*args, **kwargs)
    
    def save(self):
        return Message.objects.new_reply(self.thread, self.user,
            self.cleaned_data["content"])



try:
    from functools import wraps
except ImportError:
    from django.utils.functional import wraps  # Python 2.4 fallback.


def cached_attribute(func):
    cache_name = "_%s" % func.__name__
    @wraps(func)
    def inner(self, *args, **kwargs):
        if hasattr(self, cache_name):
            return getattr(self, cache_name)
        val = func(self, *args, **kwargs)
        setattr(self, cache_name, val)
        return val
    return inner



from django.contrib import admin
from .models import Message, Thread, UserThread

import autocomplete_light


class MessageAdmin(admin.ModelAdmin):
    list_display = ('id', 'thread')
    list_display_links = ('id',)


class ThreadAdmin(admin.ModelAdmin):
    list_display = ('id', 'subject')
    list_display_links = ('id',)


class UserThreadAdmin(admin.ModelAdmin):
    list_display = ('id',)
    list_display_links = ('id',)


admin.site.register(Message, MessageAdmin)
admin.site.register(Thread, ThreadAdmin)
admin.site.register(UserThread, UserThreadAdmin)



import os

from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import Context

from django.contrib.auth.models import User

from eldarion.test import TestCase

from user_messages.models import Thread, Message


class BaseTest(TestCase):
    def setUp(self):
        self.brosner = User.objects.create_superuser("brosner",
            "brosner@brosner.brosner", "abc123")
        self.jtauber = User.objects.create_superuser("jtauber",
            "jtauber@jtauber.jtauber", "abc123")
        if hasattr(self, "template_dirs"):
            self._old_template_dirs = settings.TEMPLATE_DIRS
            settings.TEMPLATE_DIRS = self.template_dirs
    
    def tearDown(self):
        if hasattr(self, "_old_template_dirs"):
            settings.TEMPLATE_DIRS = self._old_template_dirs


class TestMessages(BaseTest):
    def test_messages(self):
        Message.objects.new_message(self.brosner, [self.jtauber], "Really?",
            "You can't be serious")
        
        self.assertEqual(Thread.objects.inbox(self.brosner).count(), 0)
        self.assertEqual(Thread.objects.inbox(self.jtauber).count(), 1)
        
        thread = Thread.objects.inbox(self.jtauber)[0]
        
        Message.objects.new_reply(thread, self.jtauber, "Yes, I am.")
        
        self.assertEqual(Thread.objects.inbox(self.brosner).count(), 1)
        self.assertEqual(Thread.objects.inbox(self.jtauber).count(), 1)
        
        Message.objects.new_reply(thread, self.brosner, "If you say so...")
        Message.objects.new_reply(thread, self.jtauber, "Indeed I do")
        
        self.assertEqual(Thread.objects.get(pk=thread.pk).latest_message.content,
            "Indeed I do")
        self.assertEqual(Thread.objects.get(pk=thread.pk).first_message.content,
            "You can't be serious")
    
    def test_ordered(self):
        t1 = Message.objects.new_message(self.brosner, [self.jtauber], "Subject",
            "A test message").thread
        t2 = Message.objects.new_message(self.brosner, [self.jtauber], "Another",
            "Another message").thread
        t3 = Message.objects.new_message(self.brosner, [self.jtauber], "Pwnt",
            "Haha I'm spamming your inbox").thread
        self.assertEqual(Thread.ordered([t2, t1, t3]), [t3, t2, t1])


class TestMessageViews(BaseTest):
    template_dirs = [
        os.path.join(os.path.dirname(__file__), "templates")
    ]

    def test_create_message(self):
        with self.login("brosner", "abc123"):
            response = self.get("messages_inbox")
            self.assertEqual(response.status_code, 200)
            
            response = self.get("message_create")
            self.assertEqual(response.status_code, 200)
            
            data = {
                "subject": "The internet is down.",
                "content": "Does this affect any of our sites?",
                "to_user": str(self.jtauber.id)
            }
            
            response = self.post("message_create", data=data)
            self.assertEqual(response.status_code, 302)
            
            self.assertEqual(Thread.objects.inbox(self.jtauber).count(), 1)
            self.assertEqual(Thread.objects.inbox(self.brosner).count(), 0)
            
            response = self.get("message_create", user_id=self.jtauber.id)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "selected=\"selected\">jtauber</option>")
            
            thread_id = Thread.objects.inbox(self.jtauber).get().id
            
            response = self.get("messages_thread_detail", thread_id=thread_id)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "Does this affect any of our sites?")
            
        with self.login("jtauber", "abc123"):
            response = self.get("messages_inbox")
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "Does this affect")
            
            response = self.get("messages_thread_detail", thread_id=thread_id)
            self.assertContains(response, "Does this affect")
            
            response = self.post("messages_thread_delete", thread_id=thread_id)
            self.assertEqual(response.status_code, 302)
            self.assertEqual(Thread.objects.inbox(self.jtauber).count(), 0)
            
            data = {
                "content": "Nope, the internet being down doesn't affect us.",
            }
            
            response = self.post("messages_thread_detail", thread_id=thread_id, data=data)
            self.assertEqual(response.status_code, 302)
            self.assertEqual(Thread.objects.inbox(self.brosner).count(), 1)
            self.assertEqual(
                Thread.objects.inbox(self.brosner).get().messages.count(),
                2
            )
            self.assertEqual(Thread.objects.unread(self.jtauber).count(), 0)
    
    def test_urls(self):
        self.assertEqual(reverse("message_create", args=[10]), "/create/10/")

class TestTemplateTags(BaseTest):
    def test_unread(self):
        thread = Message.objects.new_message(self.brosner, [self.jtauber], "Why did you the internet?", "I demand to know.").thread
        tmpl = """{% load user_messages_tags %}{% if thread|unread:user %}UNREAD{% else %}READ{% endif %}"""
        self.assert_renders(
            tmpl,
            Context({"thread": thread, "user": self.jtauber}),
            "UNREAD"
        )
        self.assert_renders(
            tmpl,
            Context({"thread": thread, "user": self.brosner}),
            "READ",
        )
